@@ -34,7 +34,7 @@ def test_float_rtl(assert_pixels):
         __________
     ''', '''
       <style>
-        @page { size: 10px 5px }
+        @page { size: 10px 5Px }
       </style>
       <div style="direction: rtl">
         <img style="float: left" src="pattern.png">
@@ -119,7 +119,7 @@ def test_float_inline_block_rtl(assert_pixels):
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px }
+        @page { size: 15Px 5px }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
       </style>
@@ -185,7 +185,7 @@ def test_float_inline_table(assert_pixels):
         _______________
     ''', '''
       <style>
-        @page { size: 15px 5px }
+        @page { size: 15PX 5px }
         table { display: inline-table }
         body { font-family: weasyprint; font-size: 2px; line-height: 1;
                color: lime }
@@ -905,3 +905,101 @@ def test_float_split_clear_empty(assert_pixels):
         </style>
         <div>bb bb bb bb bb</div>
         <article></article>''')
+
+
+@assert_no_logs
+def test_float_split_large_parent(assert_pixels):
+    assert_pixels('''
+        BBBB
+        BBBB
+        BBBB
+        BBBB
+        GGGG
+
+        BBBB
+        BBBB
+        BBBB
+        BBBB
+        GGGG
+
+        BBBB
+        BBBB
+        GGGG
+        GGGG
+        GGGG
+
+        rrrr
+        rrrr
+        GGGG
+        GGGG
+        GGGG
+    ''', '''
+        <style>
+            @page {
+                size: 4px 5px;
+            }
+            body {
+                color: red;
+                font: 2px / 1 weasyprint;
+            }
+            section {
+                background: lime;
+                height: 100px;
+            }
+            div {
+                color: blue;
+                float: left;
+            }
+            article {
+                clear: left;
+            }
+        </style>
+        <section>
+        <div>bb bb bb bb bb</div>
+        <article>aa</article>''')
+
+
+@assert_no_logs
+def test_float_split_in_stacking_context(assert_pixels):
+    assert_pixels('''
+        RRRR
+        RRRR
+        BBBB
+        BBBB
+        ____
+
+        BBBB
+        BBBB
+        RRRR
+        RRRR
+        ____
+
+        RRRR
+        RRRR
+        ____
+        ____
+        ____
+    ''', '''
+        <style>
+            @page {
+                size: 4px 5px;
+            }
+            body {
+                color: red;
+                font: 2px / 1 weasyprint;
+            }
+            section {
+                display: flow-root;
+            }
+            div {
+                color: blue;
+                float: left;
+            }
+            article {
+                clear: left;
+            }
+        </style>
+        <section>
+        aa
+        <div>bb bb</div>
+        cc cc''')
