@@ -19,12 +19,13 @@ from PIL import Image
 
 from weasyprint import CSS, HTML, __main__, default_url_fetcher
 from weasyprint.pdf.anchors import resolve_links
-from weasyprint.pdf.metadata import generate_rdf_metadata
-from weasyprint.urls import path2url
 
 from .draw import parse_pixels
 from .testing_utils import FakeHTML, assert_no_logs, capture_logs, resource_path
 
+from weasyprint.urls import (  # isort:skip
+    FatalURLFetchingError, URLFetcher, URLFetcherResponse, URLFetchingError, path2url)
+
 try:
     # Available in Python 3.11+
     from contextlib import chdir
@@ -109,6 +110,62 @@ def _run(args, stdin=b''):
     return stdout.getvalue()
 
 
+def _gzip_compress(data):
+    file_obj = io.BytesIO()
+    gzip_file = gzip.GzipFile(fileobj=file_obj, mode='wb')
+    gzip_file.write(data)
+    gzip_file.close()
+    return file_obj.getvalue()
+
+
+@contextlib.contextmanager
+def http_server():
+    handlers = {
+        '/gzip': lambda env: (
+            (_gzip_compress(b'<html test=ok>'), {'Content-Encoding': 'gzip'})
+            if 'gzip' in env.get('HTTP_ACCEPT_ENCODING', '') else
+            (b'<html test=accept-encoding-header-fail>', {})
+        ),
+        '/deflate': lambda env: (
+            (zlib.compress(b'<html test=ok>'), {'Content-Encoding': 'deflate'})
+            if 'deflate' in env.get('HTTP_ACCEPT_ENCODING', '') else
+            (b'<html test=accept-encoding-header-fail>', {})
+        ),
+        '/raw-deflate': lambda env: (
+            # Remove zlib header and checksum
+            (zlib.compress(b'<html test=ok>')[2:-4], {'Content-Encoding': 'deflate'})
+            if 'deflate' in env.get('HTTP_ACCEPT_ENCODING', '') else
+            (b'<html test=accept-encoding-header-fail>', {})
+        ),
+        '/redirect': lambda env: (b'', {'Location': '/gzip'}),
+    }
+
+    def wsgi_app(environ, start_response):
+        handler = handlers.get(environ['PATH_INFO'])
+        if handler:
+            response, headers = handler(environ)
+            status = '301 Moved Permanently' if 'Location' in headers else '200 OK'
+            headers = [(str(name), str(value)) for name, value in headers.items()]
+        else:  # pragma: no cover
+            status = '404 Not Found'
+            response = b''
+            headers = []
+        start_response(status, headers)
+        return [response]
+
+    # Port 0: let the OS pick an available port number
+    # https://stackoverflow.com/a/1365284/1162888
+    server = wsgiref.simple_server.make_server('127.0.0.1', 0, wsgi_app)
+    _host, port = server.socket.getsockname()
+    thread = threading.Thread(target=server.serve_forever)
+    thread.start()
+    try:
+        yield f'http://127.0.0.1:{port}'
+    finally:
+        server.shutdown()
+        thread.join()
+
+
 class FakeFile:
     def __init__(self):
         self.chunks = []
@@ -244,7 +301,7 @@ def check_png_pattern(assert_pixels_equal, png_bytes, x2=False, blank=False,
         '''
     image = Image.open(io.BytesIO(png_bytes))
     width, height, pixels = parse_pixels(expected_pixels)
-    assert_pixels_equal(width, height, image.getdata(), pixels)
+    assert_pixels_equal(width, height, image.get_flattened_data(), pixels)
 
 
 @assert_no_logs
@@ -460,9 +517,10 @@ def test_command_line_render(tmp_path):
     ('3u', '1.7'),
     ('4e', '2.0'),
 ])
+@assert_no_logs
 def test_pdfa(version, pdf_version):
     stdout = _run(
-        f'--pdf-variant=pdf/a-{version} --uncompressed-pdf - -', b'test')
+        f'--pdf-variant=pdf/a-{version} --uncompressed-pdf - -', b'<html lang=en>test')
     assert f'PDF-{pdf_version}'.encode() in stdout
     assert f'part="{version[0]}"'.encode() in stdout
 
@@ -473,8 +531,9 @@ def test_pdfa(version, pdf_version):
     ('3u', '1.7'),
     ('4e', '2.0'),
 ])
+@assert_no_logs
 def test_pdfa_compressed(version, pdf_version):
-    stdout = _run(f'--pdf-variant=pdf/a-{version} - -', b'test')
+    stdout = _run(f'--pdf-variant=pdf/a-{version} - -', b'<html lang=en>test')
     assert f'PDF-{pdf_version}'.encode() in stdout
 
 
@@ -488,8 +547,10 @@ def test_pdfa1b_cidset():
     (1, '1.7'),
     (2, '2.0'),
 ])
+@assert_no_logs
 def test_pdfua(version, pdf_version):
-    stdout = _run(f'--pdf-variant=pdf/ua-{version} --uncompressed-pdf - -', b'test')
+    stdout = _run(
+        f'--pdf-variant=pdf/ua-{version} --uncompressed-pdf - -', b'<html lang=en>test')
     assert f'PDF-{pdf_version}'.encode() in stdout
     assert f'part="{version}"'.encode() in stdout
 
@@ -498,27 +559,32 @@ def test_pdfua(version, pdf_version):
     (1, '1.7'),
     (2, '2.0'),
 ])
+@assert_no_logs
 def test_pdfua_compressed(version, pdf_version):
-    stdout = _run(f'--pdf-variant=pdf/ua-{version} - -', b'test')
+    stdout = _run(f'--pdf-variant=pdf/ua-{version} - -', b'<html lang=en>test')
     assert f'PDF-{pdf_version}'.encode() in stdout
 
 
+@assert_no_logs
 def test_pdf_tags():
-    stdout = _run('--pdf-tags --uncompressed-pdf - -', b'<article>test')
+    stdout = _run('--pdf-tags --uncompressed-pdf - -', b'<html lang=en><article>test')
     assert b'/StructTreeRoot' in stdout
     assert b'/Art' in stdout
 
 
+@assert_no_logs
 def test_pdf_identifier():
     stdout = _run('--pdf-identifier=abc --uncompressed-pdf - -', b'test')
     assert b'abc' in stdout
 
 
+@assert_no_logs
 def test_pdf_version():
     stdout = _run('--pdf-version=1.4 --uncompressed-pdf - -', b'test')
     assert b'PDF-1.4' in stdout
 
 
+@assert_no_logs
 def test_pdf_custom_metadata():
     stdout = _run(
         '--custom-metadata --uncompressed-pdf - -',
@@ -527,6 +593,7 @@ def test_pdf_custom_metadata():
     assert b'value' in stdout
 
 
+@assert_no_logs
 def test_bad_pdf_custom_metadata():
     stdout = _run(
         '--custom-metadata --uncompressed-pdf - -',
@@ -534,6 +601,7 @@ def test_bad_pdf_custom_metadata():
     assert b'value' not in stdout
 
 
+@assert_no_logs
 def test_partial_pdf_custom_metadata():
     stdout = _run(
         '--custom-metadata --uncompressed-pdf - -',
@@ -542,22 +610,26 @@ def test_partial_pdf_custom_metadata():
     assert b'value' in stdout
 
 
+@assert_no_logs
 def test_pdf_srgb():
     stdout = _run('--srgb --uncompressed-pdf - -', b'test')
     assert b'sRGB' in stdout
 
 
+@assert_no_logs
 def test_pdf_no_srgb():
     stdout = _run('--uncompressed-pdf - -', b'test')
     assert b'sRGB' not in stdout
 
 
+@assert_no_logs
 def test_pdf_font_name():
     # Regression test for #2396.
     stdout = _run('--uncompressed-pdf - -', b'<div style="font-family:weasyprint">test')
     assert b'+weasyprint/' in stdout
 
 
+@assert_no_logs
 def test_to_unicode():
     # Regression test for #2388.
     stdout = _run('--uncompressed-pdf --full-fonts - -', b'test')
@@ -567,6 +639,7 @@ def test_to_unicode():
         assert int(match) <= 100
 
 
+@assert_no_logs
 def test_to_unicode_rtl():
     # Regression test for #378.
     stdout = _run(
@@ -576,6 +649,49 @@ def test_to_unicode_rtl():
     assert b'<00d0> <0628>' in stdout
 
 
+@assert_no_logs
+def test_no_redirect():
+    with http_server() as root_url:
+        _run(f'--no-http-redirect {root_url}/gzip -')
+        with pytest.raises(URLFetchingError):
+            _run(f'--no-http-redirect {root_url}/bad -')
+        with pytest.raises(URLFetchingError):
+            _run(f'--no-http-redirect {root_url}/redirect -')
+        with capture_logs() as logs:
+            _run(
+                '--no-http-redirect - -',
+                f'<link rel=stylesheet href="{root_url}/redirect">'.encode())
+        assert len(logs) == 1
+        assert 'Failed to load stylesheet' in logs[0]
+
+
+@assert_no_logs
+def test_fail_on_error():
+    with http_server() as root_url:
+        _run(f'--fail-on-http-error {root_url}/gzip -')
+        _run(f'--fail-on-http-error {root_url}/redirect -')
+        with pytest.raises(FatalURLFetchingError):
+            _run(f'--fail-on-http-error {root_url}/bad -')
+        with pytest.raises(FatalURLFetchingError):
+            _run(
+                '--fail-on-http-error - -',
+                f'<link rel=stylesheet href="{root_url}/bad">'.encode())
+
+
+@assert_no_logs
+def test_no_redirect_fail_on_error():
+    with http_server() as root_url:
+        _run(f'--no-http-redirect --fail-on-http-error {root_url}/gzip -')
+        with pytest.raises(FatalURLFetchingError):
+            _run(f'--no-http-redirect --fail-on-http-error {root_url}/redirect -')
+        with pytest.raises(FatalURLFetchingError):
+            _run(f'--no-http-redirect --fail-on-http-error {root_url}/bad -')
+        with pytest.raises(FatalURLFetchingError):
+            _run(
+                '--no-http-redirect --fail-on-http-error - -',
+                f'<link rel=stylesheet href="{root_url}/bad">'.encode())
+
+
 @assert_no_logs
 @pytest.mark.parametrize('command', [
     '- -',
@@ -1132,8 +1248,9 @@ def test_links_12():
 uses_relative.append('weasyprint-custom')
 
 
+@pytest.mark.filterwarnings('ignore')
 @assert_no_logs
-def test_url_fetcher(assert_pixels_equal):
+def test_deprecated_url_fetcher(assert_pixels_equal):
     path = resource_path('pattern.png')
     pattern_png = path.read_bytes()
 
@@ -1174,7 +1291,6 @@ def test_url_fetcher(assert_pixels_equal):
     test('<style>@import "weasyprint-custom:foo/bar.css";</style><body>')
     test('<style>@import url(weasyprint-custom:foo/bar.css);</style><body>')
     test('<style>@import url("weasyprint-custom:foo/bar.css");</style><body>')
-    test('<link rel=stylesheet href="weasyprint-custom:foo/bar.css"><body>')
 
     with capture_logs() as logs:
         test('<body><img src="custom:foo/bar">', blank=True)
@@ -1197,6 +1313,108 @@ def test_url_fetcher(assert_pixels_equal):
         url_fetcher=fetcher_2).render()
 
 
+class Fetcher(URLFetcher):
+    def fetch(self, url, headers=None):
+        if url.startswith('fatal:'):
+            raise FatalURLFetchingError('Fatal error')
+        elif 'forbidden' in url:
+            raise FatalURLFetchingError('Forbidden URL')
+        elif url.startswith('bad:'):
+            raise ValueError('Bad protocol')
+        elif url == 'redirect:':
+            return self.fetch('weasyprint-custom:foo/bar.css')
+        elif url == 'weasyprint-custom:foo/%C3%A9_%e9_pattern':
+            pattern_png = resource_path('pattern.png').read_bytes()
+            return URLFetcherResponse(url, pattern_png, {'Content-Type': 'image/png'})
+        elif url == 'weasyprint-custom:foo/bar.css':
+            return URLFetcherResponse(
+                url, 'body { background: url(é_%e9_pattern)'.encode('iso-8859-15'),
+                {'Content-Type': 'text/css;charset=iso-8859-15'})
+        elif url == 'weasyprint-custom:foo/bar.no':
+            return URLFetcherResponse(
+                url, 'body { background: red }', {'Content-Type': 'text/no'})
+        else:
+            return super().fetch(url, headers)
+
+
+@pytest.mark.parametrize('html', [
+    '<body><img src="pattern.png">',
+    f'<body><img src="{resource_path("pattern.png").as_uri()}">',
+    f'<body><img src="{resource_path("pattern.png").as_uri()}?ignored">',
+    '<body><img src="weasyprint-custom:foo/é_%e9_pattern">',
+    '<body style="background: url(weasyprint-custom:foo/é_%e9_pattern)">',
+    '<body><li style="list-style: inside url(weasyprint-custom:foo/é_%e9_pattern)">',
+    '<link rel=stylesheet href="weasyprint-custom:foo/bar.css"><body>',
+    '<link rel=stylesheet href="redirect:"><body>',
+    '<style>@import "weasyprint-custom:foo/bar.css";</style><body>',
+    '<style>@import url(weasyprint-custom:foo/bar.css);</style><body>',
+    '<style>@import url("weasyprint-custom:foo/bar.css");</style><body>',
+])
+@assert_no_logs
+def test_url_fetcher(html, assert_pixels_equal):
+    base_url = str(resource_path('dummy.html'))
+    css = CSS(string='@page{size:8px;margin:2px} body{font-size:0}', base_url=base_url)
+    html = FakeHTML(string=html, url_fetcher=Fetcher(), base_url=base_url)
+    check_png_pattern(assert_pixels_equal, html.write_png(stylesheets=[css]))
+
+
+@assert_no_logs
+def test_url_fetcher_default(assert_pixels_equal):
+    html = '<body><img src="pattern.png">'
+    base_url = str(resource_path('dummy.html'))
+    css = CSS(string='@page{size:8px;margin:2px} body{font-size:0}', base_url=base_url)
+    html = FakeHTML(string=html, url_fetcher=URLFetcher(), base_url=base_url)
+    check_png_pattern(assert_pixels_equal, html.write_png(stylesheets=[css]))
+
+
+@assert_no_logs
+def test_url_fetcher_bad_image(assert_pixels_equal):
+    string = '<body><img src="custom:foo/bar">'
+    css = CSS(string='@page { size: 8px; margin: 2px } body { font-size: 0 }')
+    with capture_logs() as logs:
+        html = FakeHTML(string=string, url_fetcher=Fetcher())
+        png = html.write_png(stylesheets=[css])
+    assert len(logs) == 1
+    assert logs[0].startswith("ERROR: Failed to load image at 'custom:foo/bar'")
+    check_png_pattern(assert_pixels_equal, png, blank=True)
+
+
+@assert_no_logs
+def test_url_fetcher_bad_stylesheet(assert_pixels_equal):
+    string = (
+        '<link rel=stylesheet href="weasyprint-custom:foo/bar.css">'
+        '<link rel=stylesheet href="weasyprint-custom:foo/bar.no"><body>')
+    with capture_logs() as logs:
+        FakeHTML(string=string, url_fetcher=Fetcher()).write_png()
+    assert len(logs) == 1
+    assert logs[0].startswith('ERROR: Unsupported stylesheet type text/no')
+
+
+@assert_no_logs
+def test_url_fetcher_non_fatal_error(assert_pixels_equal):
+    string = '<link rel=stylesheet href="bad:foo/bar.no"><body>'
+    with capture_logs() as logs:
+        FakeHTML(string=string, url_fetcher=Fetcher()).write_png()
+    assert len(logs) == 1
+    assert 'Bad protocol' in logs[0]
+
+
+@assert_no_logs
+def test_url_fetcher_fatal_error(assert_pixels_equal):
+    string = '<link rel=stylesheet href="fatal:foo/bar.no"><body>'
+    with pytest.raises(FatalURLFetchingError, match='Fatal error'):
+        FakeHTML(string=string, url_fetcher=Fetcher()).write_png()
+
+
+@assert_no_logs
+def test_url_fetcher_default_fail_on_errors(assert_pixels_equal):
+    html = '<body><img src="unknown.png">'
+    base_url = str(resource_path('dummy.html'))
+    url_fetcher = URLFetcher(fail_on_errors=True)
+    with pytest.raises(FatalURLFetchingError, match='Error fetching'):
+        FakeHTML(string=html, url_fetcher=url_fetcher, base_url=base_url).write_png()
+
+
 def assert_meta(html, **meta):
     meta.setdefault('title', None)
     meta.setdefault('authors', [])
@@ -1208,7 +1426,7 @@ def assert_meta(html, **meta):
     meta.setdefault('attachments', [])
     meta.setdefault('lang', None)
     meta.setdefault('custom', {})
-    meta.setdefault('generate_rdf_metadata', generate_rdf_metadata)
+    meta.setdefault('xmp_metadata', [])
     assert vars(FakeHTML(string=html).render().metadata) == meta
 
 
@@ -1285,64 +1503,45 @@ def test_html_meta_4():
 
 @assert_no_logs
 def test_http():
-    def gzip_compress(data):
-        file_obj = io.BytesIO()
-        gzip_file = gzip.GzipFile(fileobj=file_obj, mode='wb')
-        gzip_file.write(data)
-        gzip_file.close()
-        return file_obj.getvalue()
-
-    @contextlib.contextmanager
-    def http_server(handlers):
-        def wsgi_app(environ, start_response):
-            handler = handlers.get(environ['PATH_INFO'])
-            if handler:
-                status = '200 OK'
-                response, headers = handler(environ)
-                headers = [(str(name), str(value)) for name, value in headers]
-            else:  # pragma: no cover
-                status = '404 Not Found'
-                response = b''
-                headers = []
-            start_response(status, headers)
-            return [response]
-
-        # Port 0: let the OS pick an available port number
-        # https://stackoverflow.com/a/1365284/1162888
-        server = wsgiref.simple_server.make_server('127.0.0.1', 0, wsgi_app)
-        _host, port = server.socket.getsockname()
-        thread = threading.Thread(target=server.serve_forever)
-        thread.start()
-        try:
-            yield f'http://127.0.0.1:{port}'
-        finally:
-            server.shutdown()
-            thread.join()
-
-    with http_server({
-        '/gzip': lambda env: (
-            (gzip_compress(b'<html test=ok>'), [('Content-Encoding', 'gzip')])
-            if 'gzip' in env.get('HTTP_ACCEPT_ENCODING', '') else
-            (b'<html test=accept-encoding-header-fail>', [])
-        ),
-        '/deflate': lambda env: (
-            (zlib.compress(b'<html test=ok>'),
-             [('Content-Encoding', 'deflate')])
-            if 'deflate' in env.get('HTTP_ACCEPT_ENCODING', '') else
-            (b'<html test=accept-encoding-header-fail>', [])
-        ),
-        '/raw-deflate': lambda env: (
-            # Remove zlib header and checksum
-            (zlib.compress(b'<html test=ok>')[2:-4],
-             [('Content-Encoding', 'deflate')])
-            if 'deflate' in env.get('HTTP_ACCEPT_ENCODING', '') else
-            (b'<html test=accept-encoding-header-fail>', [])
-        ),
-    }) as root_url:
+    with http_server() as root_url:
         assert HTML(f'{root_url}/gzip').etree_element.get('test') == 'ok'
         assert HTML(f'{root_url}/deflate').etree_element.get('test') == 'ok'
-        assert HTML(
-            f'{root_url}/raw-deflate').etree_element.get('test') == 'ok'
+        assert HTML(f'{root_url}/raw-deflate').etree_element.get('test') == 'ok'
+        assert HTML(f'{root_url}/redirect').etree_element.get('test') == 'ok'
+
+        url_fetcher = URLFetcher()
+        HTML(f'{root_url}/redirect', url_fetcher=url_fetcher).render()
+        with capture_logs() as logs:
+            HTML(
+                string=f'<link rel=stylesheet href="{root_url}/redirect">',
+                url_fetcher=url_fetcher).render()
+        assert len(logs) == 1
+        assert 'Unsupported stylesheet' in logs[0]
+
+        url_fetcher = URLFetcher(allow_redirects=False)
+        with pytest.raises(URLFetchingError, match='301'):
+            HTML(f'{root_url}/redirect', url_fetcher=url_fetcher).render()
+        with capture_logs() as logs:
+            HTML(
+                string=f'<link rel=stylesheet href="{root_url}/redirect">',
+                url_fetcher=url_fetcher).render()
+        assert len(logs) == 1
+        assert '301' in logs[0]
+
+        url_fetcher = URLFetcher(fail_on_errors=True)
+        HTML(f'{root_url}/redirect', url_fetcher=url_fetcher).render()
+        with pytest.raises(FatalURLFetchingError, match='Error fetching'):
+            HTML(
+                string=f'<img src="{root_url}/bad">',
+                url_fetcher=url_fetcher).render()
+
+        url_fetcher = URLFetcher(allow_redirects=False, fail_on_errors=True)
+        with pytest.raises(FatalURLFetchingError, match='Error fetching'):
+            HTML(f'{root_url}/redirect', url_fetcher=url_fetcher).render()
+        with pytest.raises(FatalURLFetchingError, match='Error fetching'):
+            HTML(
+                string=f'<link rel=stylesheet href="{root_url}/redirect">',
+                url_fetcher=url_fetcher).render()
 
 
 @assert_no_logs
