@@ -5,7 +5,6 @@ importing sub-modules.
 
 """
 
-import contextlib
 from datetime import datetime
 from os.path import getctime, getmtime
 from pathlib import Path
@@ -15,9 +14,9 @@ import cssselect2
 import tinycss2
 import tinyhtml5
 
-VERSION = __version__ = '67.0'
+VERSION = __version__ = '68.0'
 
-#: Default values for command-line and Python API options. See
+#: Default values for command-line and Python API rendering options. See
 #: :func:`__main__.main` to learn more about specific options for
 #: command-line.
 #:
@@ -67,14 +66,15 @@ VERSION = __version__ = '67.0'
 #:     images are temporarily stored.
 DEFAULT_OPTIONS = {
     'stylesheets': None,
-    'media_type': 'print',
     'attachments': None,
+    'attachment_relationships': None,
     'pdf_identifier': None,
     'pdf_variant': None,
     'pdf_version': None,
     'pdf_forms': None,
     'pdf_tags': False,
     'uncompressed_pdf': False,
+    'xmp_metadata': None,
     'custom_metadata': False,
     'presentational_hints': False,
     'srgb': False,
@@ -92,8 +92,7 @@ __all__ = [
 
 
 # Import after setting the version, as the version is used in other modules
-from .urls import (  # noqa: I001, E402
-    fetch, default_url_fetcher, path2url, ensure_url, url_is_absolute)
+from .urls import URLFetcher, default_url_fetcher, select_source  # noqa: I001, E402
 from .logger import LOGGER, PROGRESS_LOGGER  # noqa: E402
 # Some imports are at the end of the file (after the CSS class)
 # to work around circular imports.
@@ -150,9 +149,7 @@ class HTML:
         :term:`file objects <file object>`.
     :type url_fetcher: :term:`callable`
     :param url_fetcher:
-        A function or other callable with the same signature as
-        :func:`default_url_fetcher` called to fetch external resources such as
-        stylesheets and images. (See :ref:`URL Fetchers`.)
+        An instance of :class:`urls.URLFetcher`. (See :ref:`URL Fetchers`.)
     :param str media_type:
         The media type to use for ``@media``. Defaults to ``'print'``.
         **Note:** In some cases like ``HTML(string=foo)`` relative URLs will be
@@ -161,25 +158,24 @@ class HTML:
     """
     def __init__(self, guess=None, filename=None, url=None, file_obj=None,
                  string=None, encoding=None, base_url=None,
-                 url_fetcher=default_url_fetcher, media_type='print'):
+                 url_fetcher=None, media_type='print'):
         PROGRESS_LOGGER.info(
             'Step 1 - Fetching and parsing HTML - %s',
             guess or filename or url or
             getattr(file_obj, 'name', 'HTML string'))
         if isinstance(base_url, Path):
             base_url = str(base_url)
-        result = _select_source(
+        if url_fetcher is None:
+            url_fetcher = URLFetcher()
+        result = select_source(
             guess, filename, url, file_obj, string, base_url, url_fetcher)
-        with result as (source_type, source, base_url, protocol_encoding):
-            if isinstance(source, str):
-                result = tinyhtml5.parse(source, namespace_html_elements=False)
-            else:
-                kwargs = {'namespace_html_elements': False}
-                if protocol_encoding is not None:
-                    kwargs['transport_encoding'] = protocol_encoding
-                if encoding is not None:
-                    kwargs['override_encoding'] = encoding
-                result = tinyhtml5.parse(source, **kwargs)
+        with result as (file_obj, base_url, protocol_encoding, _):
+            kwargs = {'namespace_html_elements': False}
+            if protocol_encoding is not None:
+                kwargs['transport_encoding'] = protocol_encoding
+            if encoding is not None:
+                kwargs['override_encoding'] = encoding
+            result = tinyhtml5.parse(file_obj, **kwargs)
         self.base_url = _find_base_url(result, base_url)
         self.url_fetcher = url_fetcher
         self.media_type = media_type
@@ -286,28 +282,26 @@ class CSS:
     of :class:`HTML` objects.
 
     """
-    def __init__(self, guess=None, filename=None, url=None, file_obj=None,
-                 string=None, encoding=None, base_url=None,
-                 url_fetcher=default_url_fetcher, _check_mime_type=False,
+    def __init__(self, guess=None, filename=None, url=None, file_obj=None, string=None,
+                 encoding=None, base_url=None, url_fetcher=None, _check_mime_type=False,
                  media_type='print', font_config=None, counter_style=None,
                  color_profiles=None, matcher=None, page_rules=None, layers=None,
                  layer=None):
         PROGRESS_LOGGER.info(
             'Step 2 - Fetching and parsing CSS - %s',
             filename or url or getattr(file_obj, 'name', 'CSS string'))
-        result = _select_source(
-            guess, filename, url, file_obj, string,
-            base_url=base_url, url_fetcher=url_fetcher,
-            check_css_mime_type=_check_mime_type)
-        with result as (source_type, source, base_url, protocol_encoding):
-            if source_type == 'file_obj':
-                source = source.read()
-            if isinstance(source, str):
-                # unicode, no encoding
-                stylesheet = tinycss2.parse_stylesheet(source)
+        if url_fetcher is None:
+            url_fetcher = URLFetcher()
+        result = select_source(
+            guess, filename, url, file_obj, string, base_url=base_url,
+            url_fetcher=url_fetcher, check_css_mime_type=_check_mime_type)
+        with result as (file_obj, base_url, protocol_encoding, mime_type):
+            css = file_obj.read()
+            if isinstance(css, str):
+                stylesheet = tinycss2.parse_stylesheet(css)
             else:
-                stylesheet, encoding = tinycss2.parse_stylesheet_bytes(
-                    source, environment_encoding=encoding,
+                stylesheet, _ = tinycss2.parse_stylesheet_bytes(
+                    css, environment_encoding=encoding,
                     protocol_encoding=protocol_encoding)
         self.base_url = base_url
         self.matcher = matcher or cssselect2.Matcher()
@@ -347,10 +341,12 @@ class Attachment:
 
     """
     def __init__(self, guess=None, filename=None, url=None, file_obj=None,
-                 string=None, base_url=None, url_fetcher=default_url_fetcher,
-                 name=None, description=None, created=None, modified=None,
+                 string=None, base_url=None, url_fetcher=None, name=None,
+                 description=None, created=None, modified=None,
                  relationship='Unspecified'):
-        self.source = _select_source(
+        if url_fetcher is None:
+            url_fetcher = URLFetcher()
+        self.source = select_source(
             guess, filename, url, file_obj, string, base_url=base_url,
             url_fetcher=url_fetcher)
         self.name = name
@@ -372,70 +368,6 @@ class Attachment:
         self.modified = modified
 
 
-@contextlib.contextmanager
-def _select_source(guess=None, filename=None, url=None, file_obj=None,
-                   string=None, base_url=None, url_fetcher=default_url_fetcher,
-                   check_css_mime_type=False):
-    """If only one input is given, return it with normalized ``base_url``."""
-    if base_url is not None:
-        base_url = ensure_url(base_url)
-
-    selected_params = [
-        param for param in (guess, filename, url, file_obj, string) if
-        param is not None]
-    if len(selected_params) != 1:
-        source = ', '.join(selected_params) or 'nothing'
-        raise TypeError(f'Expected exactly one source, got {source}')
-    elif guess is not None:
-        if hasattr(guess, 'read'):
-            type_ = 'file_obj'
-        elif isinstance(guess, Path):
-            type_ = 'filename'
-        elif url_is_absolute(guess):
-            type_ = 'url'
-        else:
-            type_ = 'filename'
-        result = _select_source(
-            base_url=base_url, url_fetcher=url_fetcher,
-            check_css_mime_type=check_css_mime_type,
-            **{type_: guess})
-        with result as result:
-            yield result
-    elif filename is not None:
-        if base_url is None:
-            base_url = path2url(filename)
-        with open(filename, 'rb') as file_obj:
-            yield 'file_obj', file_obj, base_url, None
-    elif url is not None:
-        with fetch(url_fetcher, url) as result:
-            if check_css_mime_type and result['mime_type'] != 'text/css':
-                LOGGER.error(
-                    'Unsupported stylesheet type %s for %s',
-                    result['mime_type'], result['redirected_url'])
-                yield 'string', '', base_url, None
-            else:
-                proto_encoding = result.get('encoding')
-                if base_url is None:
-                    base_url = result.get('redirected_url', url)
-                if 'string' in result:
-                    yield 'string', result['string'], base_url, proto_encoding
-                else:
-                    yield (
-                        'file_obj', result['file_obj'], base_url,
-                        proto_encoding)
-    elif file_obj is not None:
-        if base_url is None:
-            # filesystem file-like objects have a 'name' attribute.
-            name = getattr(file_obj, 'name', None)
-            # Some streams have a .name like '<stdin>', not a filename.
-            if name and not name.startswith('<'):
-                base_url = ensure_url(name)
-        yield 'file_obj', file_obj, base_url, None
-    else:
-        assert string is not None
-        yield 'string', string, base_url, None
-
-
 # Work around circular imports.
 from .css import preprocess_stylesheet  # noqa: I001, E402
 from .html import (  # noqa: E402
