@@ -1,67 +1,62 @@
-"""
-    weasyprint.formatting_structure.boxes
-    -------------------------------------
+"""Classes for all types of boxes in the CSS formatting structure / box model.
 
-    Classes for all types of boxes in the CSS formatting structure / box model.
+See https://www.w3.org/TR/CSS21/visuren.html
 
-    See http://www.w3.org/TR/CSS21/visuren.html
+Names are the same as in CSS 2.1 with the exception of ``TextBox``. In
+WeasyPrint, any text is in a ``TextBox``. What CSS calls anonymous inline boxes
+are text boxes but not all text boxes are anonymous inline boxes.
 
-    Names are the same as in CSS 2.1 with the exception of ``TextBox``. In
-    WeasyPrint, any text is in a ``TextBox``. What CSS calls anonymous
-    inline boxes are text boxes but not all text boxes are anonymous
-    inline boxes.
+See https://www.w3.org/TR/CSS21/visuren.html#anonymous
 
-    See http://www.w3.org/TR/CSS21/visuren.html#anonymous
+Abstract classes, should not be instantiated:
 
-    Abstract classes, should not be instantiated:
+* Box
+* BlockLevelBox
+* InlineLevelBox
+* BlockContainerBox
+* ReplacedBox
+* ParentBox
+* AtomicInlineLevelBox
 
-    * Box
-    * BlockLevelBox
-    * InlineLevelBox
-    * BlockContainerBox
-    * ReplacedBox
-    * ParentBox
-    * AtomicInlineLevelBox
+Concrete classes:
 
-    Concrete classes:
+* PageBox
+* BlockBox
+* InlineBox
+* InlineBlockBox
+* BlockReplacedBox
+* InlineReplacedBox
+* TextBox
+* LineBox
+* Various table-related Box subclasses
 
-    * PageBox
-    * BlockBox
-    * InlineBox
-    * InlineBlockBox
-    * BlockReplacedBox
-    * InlineReplacedBox
-    * TextBox
-    * LineBox
-    * Various table-related Box subclasses
+All concrete box classes whose name contains "Inline" or "Block" have one of
+the following "outside" behavior:
 
-    All concrete box classes whose name contains "Inline" or "Block" have
-    one of the following "outside" behavior:
+* Block-level (inherits from :class:`BlockLevelBox`)
+* Inline-level (inherits from :class:`InlineLevelBox`)
 
-    * Block-level (inherits from :class:`BlockLevelBox`)
-    * Inline-level (inherits from :class:`InlineLevelBox`)
+and one of the following "inside" behavior:
 
-    and one of the following "inside" behavior:
+* Block container (inherits from :class:`BlockContainerBox`)
+* Inline content (InlineBox and :class:`TextBox`)
+* Replaced content (inherits from :class:`ReplacedBox`)
 
-    * Block container (inherits from :class:`BlockContainerBox`)
-    * Inline content (InlineBox and :class:`TextBox`)
-    * Replaced content (inherits from :class:`ReplacedBox`)
+… with various combinasions of both.
 
-    ... with various combinasions of both.
-
-    See respective docstrings for details.
+See respective docstrings for details.
 
 """
 
 import itertools
 
-from ..css import computed_from_cascaded
+from ..css import AnonymousStyle
 
 
 class Box:
     """Abstract base class for all boxes."""
     # Definitions for the rules generating anonymous table boxes
-    # http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
+    # https://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
     proper_table_child = False
     internal_table_or_caption = False
     tabular_container = False
@@ -73,27 +68,43 @@ class Box:
     # Default, may be overriden on instances.
     is_table_wrapper = False
     is_flex_item = False
+    is_grid_item = False
     is_for_root_element = False
     is_column = False
     is_leader = False
-    is_attachment = False
+    is_outside_marker = False
 
     # Other properties
     transformation_matrix = None
     bookmark_label = None
     string_set = None
-    download_name = None
     footnote = None
+    cached_counter_values = None
+    missing_link = None
+    link_annotation = None
+    force_fragmentation = False
 
     # Default, overriden on some subclasses
     def all_children(self):
-        return ()
+        return self.children
+
+    def descendants(self, placeholders=False):
+        """A flat generator for a box, its children and descendants."""
+        yield self
+        for child in self.children:
+            if placeholders or isinstance(child, Box):
+                yield from child.descendants(placeholders)
+            else:
+                yield child
 
     def __init__(self, element_tag, style, element):
         self.element_tag = element_tag
         self.element = element
         self.style = style
         self.remove_decoration_sides = set()
+        self.children = []
+        self.first_letter_style = None
+        self.first_line_style = None
 
     def __repr__(self):
         return f'<{type(self).__name__} {self.element_tag}>'
@@ -101,8 +112,7 @@ class Box:
     @classmethod
     def anonymous_from(cls, parent, *args, **kwargs):
         """Return an anonymous box that inherits from ``parent``."""
-        style = computed_from_cascaded(
-            cascaded={}, parent_style=parent.style, element=None)
+        style = AnonymousStyle(parent.style)
         return cls(parent.element_tag, style, parent.element, *args, **kwargs)
 
     def copy(self):
@@ -126,7 +136,7 @@ class Box:
 
         """
         # Overridden in ParentBox to also translate children, if any.
-        if dx == 0 and dy == 0:
+        if dx == dy == 0:
             return
         self.position_x += dx
         self.position_y += dy
@@ -193,7 +203,7 @@ class Box:
     def hit_area(self):
         """Return the (x, y, w, h) rectangle where the box is clickable."""
         # "Border area. That's the area that hit-testing is done on."
-        # http://lists.w3.org/Archives/Public/www-style/2012Jun/0318.html
+        # https://lists.w3.org/Archives/Public/www-style/2012Jun/0318.html
         # TODO: manage the border radii, use outer_border_radii instead
         return (self.border_box_x(), self.border_box_y(),
                 self.border_width(), self.border_height())
@@ -225,15 +235,15 @@ class Box:
         height = self.border_height() - bt - bb
 
         # Fix overlapping curves
-        # See http://www.w3.org/TR/css3-background/#corner-overlap
+        # See https://www.w3.org/TR/css-backgrounds-3/#corner-overlap
         ratio = min([1] + [
             extent / sum_radii
-            for extent, sum_radii in [
+            for extent, sum_radii in (
                 (width, tlrx + trrx),
                 (width, blrx + brrx),
                 (height, tlry + blry),
                 (height, trry + brry),
-            ]
+            )
             if sum_radii > 0
         ])
         return (
@@ -294,12 +304,59 @@ class Box:
             self.is_floated() or self.is_absolutely_positioned() or
             self.is_running() or self.is_footnote())
 
+    def is_monolithic(self):
+        """Return whether this box is monolithic."""
+        # https://www.w3.org/TR/css-break-3/#monolithic
+        return (
+            isinstance(self, AtomicInlineLevelBox) or
+            isinstance(self, ReplacedBox) or
+            self.style['overflow'] in ('auto', 'scroll') or
+            (self.style['overflow'] == 'hidden' and
+             self.style['height'] != 'auto'))
+
+    def establishes_formatting_context(self):
+        """Return whether this box establishes a block formatting context."""
+        # See https://www.w3.org/TR/CSS2/visuren.html#block-formatting
+        return (
+            self.is_floated() or
+            self.is_absolutely_positioned() or
+            self.is_column or
+            (isinstance(self, BlockContainerBox) and not isinstance(self, BlockBox)) or
+            (isinstance(self, BlockBox) and self.style['overflow'] != 'visible') or
+            'flow-root' in self.style['display'])
+
     # Start and end page values for named pages
 
     def page_values(self):
         """Return start and end page values."""
         return (self.style['page'], self.style['page'])
 
+    # PDF attachments
+
+    def is_attachment(self):
+        """Return whether this link should be stored as a PDF attachment."""
+        from ..html import element_has_link_type
+
+        if self.element is not None and self.element.tag == 'a':
+            return element_has_link_type(self.element, 'attachment')
+        return False
+
+    # Forms
+
+    def is_input(self):
+        """Return whether this box is a form input."""
+        # https://html.spec.whatwg.org/multipage/forms.html#category-submit
+        if self.style['appearance'] == 'auto' and self.element is not None:
+            if self.element.tag in ('button', 'input', 'select', 'textarea'):
+                return not isinstance(self, (LineBox, TextBox))
+        return False
+
+    def is_form(self):
+        """Return whether this box is a form element."""
+        if self.element is None:
+            return False
+        return self.element.tag == 'form'
+
 
 class ParentBox(Box):
     """A box that has children."""
@@ -307,9 +364,6 @@ class ParentBox(Box):
         super().__init__(element_tag, style, element)
         self.children = tuple(children)
 
-    def all_children(self):
-        return self.children
-
     def _reset_spacing(self, side):
         """Set to 0 the margin, padding and border of ``side``."""
         self.remove_decoration_sides.add(side)
@@ -328,7 +382,7 @@ class ParentBox(Box):
     def copy_with_children(self, new_children):
         """Create a new equivalent box with given ``new_children``."""
         new_box = self.copy()
-        new_box.children = tuple(new_children)
+        new_box.children = new_children
 
         # Clear and reset removed decorations as we don't want to keep the
         # previous data, for example when a box is split between two pages.
@@ -338,19 +392,9 @@ class ParentBox(Box):
 
     def deepcopy(self):
         result = self.copy()
-        result.children = tuple(child.deepcopy() for child in self.children)
+        result.children = list(child.deepcopy() for child in self.children)
         return result
 
-    def descendants(self):
-        """A flat generator for a box, its children and descendants."""
-        yield self
-        for child in self.children:
-            if isinstance(child, ParentBox):
-                for grand_child in child.descendants():
-                    yield grand_child
-            else:
-                yield child
-
     def get_wrapped_table(self):
         """Get the table wrapped by the box."""
         assert self.is_table_wrapper
@@ -362,17 +406,38 @@ class ParentBox(Box):
 
     def page_values(self):
         start_value, end_value = super().page_values()
-        if self.children:
-            if len(self.children) == 1:
-                page_values = self.children[0].page_values()
+        # TODO: We should find Class A possible page breaks according to
+        # https://drafts.csswg.org/css-page-3/#propdef-page
+        # Keep only children in normal flow for now.
+        children = [
+            child for child in self.children if child.is_in_normal_flow()]
+        if children:
+            if len(children) == 1:
+                page_values = children[0].page_values()
                 start_value = page_values[0] or start_value
                 end_value = page_values[1] or end_value
             else:
-                start_box, end_box = self.children[0], self.children[-1]
+                start_box, end_box = children[0], children[-1]
                 start_value = start_box.page_values()[0] or start_value
                 end_value = end_box.page_values()[1] or end_value
         return start_value, end_value
 
+    def top_margin_collapses(self):
+        return not (
+            self.border_top_width or self.padding_top or
+            self.is_flex_item or self.is_grid_item or
+            self.establishes_formatting_context() or
+            self.is_table_wrapper or
+            self.is_for_root_element)
+
+    def bottom_margin_collapses(self):
+        return not (
+            self.border_bottom_width or self.padding_bottom or
+            self.is_flex_item or self.is_grid_item or
+            self.establishes_formatting_context() or
+            self.is_table_wrapper or
+            self.is_for_root_element)
+
 
 class BlockLevelBox(Box):
     """A box that participates in an block formatting context.
@@ -539,7 +604,7 @@ class InlineReplacedBox(ReplacedBox, AtomicInlineLevelBox):
 class TableBox(BlockLevelBox, ParentBox):
     """Box for elements with ``display: table``"""
     # Definitions for the rules generating anonymous table boxes
-    # http://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
+    # https://www.w3.org/TR/CSS21/tables.html#anonymous-boxes
     tabular_container = True
 
     def all_children(self):
@@ -584,9 +649,6 @@ class TableColumnGroupBox(ParentBox):
     internal_table_or_caption = True
     proper_parents = (TableBox, InlineTableBox)
 
-    # Default value. May be overriden on instances.
-    span = 1
-
     # Columns groups never have margins or paddings
     margin_top = 0
     margin_bottom = 0
@@ -603,6 +665,16 @@ class TableColumnGroupBox(ParentBox):
         return [
             cell for column in self.children for cell in column.get_cells()]
 
+    @property
+    def span(self):
+        if self.children:
+            return len(self.children)
+        else:
+            try:
+                return max(int(self.element.get('span', '').strip()), 1)
+            except ValueError:
+                return 1
+
 
 # Not really a parent box, but pretending to be removes some corner cases.
 class TableColumnBox(ParentBox):
@@ -611,9 +683,6 @@ class TableColumnBox(ParentBox):
     internal_table_or_caption = True
     proper_parents = (TableBox, InlineTableBox, TableColumnGroupBox)
 
-    # Default value. May be overriden on instances.
-    span = 1
-
     # Columns never have margins or paddings
     margin_top = 0
     margin_bottom = 0
@@ -631,15 +700,36 @@ class TableColumnBox(ParentBox):
         Is set on instances.
 
         """
+        raise NotImplementedError
+
+    @property
+    def span(self):
+        try:
+            return max(int(self.element.get('span', '').strip()), 1)
+        except ValueError:
+            return 1
 
 
 class TableCellBox(BlockContainerBox):
     """Box for elements with ``display: table-cell``"""
     internal_table_or_caption = True
 
-    # Default values. May be overriden on instances.
-    colspan = 1
-    rowspan = 1
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # HTML 4.01 gives special meaning to colspan=0
+        # https://www.w3.org/TR/html401/struct/tables.html#adef-rowspan
+        # but HTML 5 removed it
+        # https://html.spec.whatwg.org/multipage/tables.html#attr-tdth-colspan
+        # rowspan=0 is still there though.
+        try:
+            self.colspan = max(int(self.element.get('colspan', '').strip()), 1)
+        except (AttributeError, ValueError):
+            self.colspan = 1
+        try:
+            self.rowspan = max(int(self.element.get('rowspan', '').strip()), 0)
+        except (AttributeError, ValueError):
+            self.rowspan = 1
 
 
 class TableCaptionBox(BlockBox):
@@ -665,6 +755,19 @@ class PageBox(ParentBox):
     def __repr__(self):
         return f'<{type(self).__name__} {self.page_type}>'
 
+    @property
+    def bleed(self):
+        return {
+            side: self.style[f'bleed_{side}'].value
+            for side in ('top', 'right', 'bottom', 'left')}
+
+    @property
+    def bleed_area(self):
+        return (
+            -self.bleed['left'], -self.bleed['top'],
+            self.margin_width() + self.bleed['left'] + self.bleed['right'],
+            self.margin_height() + self.bleed['top'] + self.bleed['bottom'])
+
 
 class MarginBox(BlockContainerBox):
     """Box in page margins, as defined in CSS3 Paged Media"""
@@ -708,3 +811,27 @@ class InlineFlexBox(FlexContainerBox, InlineLevelBox):
     It behaves as inline on the outside and as a flex container on the inside.
 
     """
+
+
+class GridContainerBox(ParentBox):
+    """A box that contains only grid-items."""
+    def __init__(self, element_tag, style, element, children):
+        super().__init__(element_tag, style, element, children)
+        # TODO: we shouldn’t store this in the box but in the rendering context instead.
+        self.advancements = {}
+
+
+class GridBox(GridContainerBox, BlockLevelBox):
+    """A box that is both block-level and a grid container.
+
+    It behaves as block on the outside and as a grid container on the inside.
+
+    """
+
+
+class InlineGridBox(GridContainerBox, InlineLevelBox):
+    """A box that is both inline-level and a grid container.
+
+    It behaves as inline on the outside and as a grid container on the inside.
+
+    """
